using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace WindowsGame1
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;
        Model model;
        Matrix world = Matrix.CreateFromAxisAngle(new Vector3(1.0f, 1.0f, 1.0f), MathHelper.ToRadians(40.0f));
        Effect effect;

        readonly float turningSpeed = MathHelper.ToRadians(90.0f); // Radiant pro Sekunde

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);

            Window.AllowUserResizing = true;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            base.Initialize();
        }

        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                // TODO: Load any ResourceManagementMode.Automatic content

                model = content.Load<Model>(@"model");

                effect = content.Load<Effect>("effect");
                effect.CurrentTechnique = effect.Techniques["phong"];
                foreach (ModelMesh mm in model.Meshes)
                {
                    foreach(ModelMeshPart mmp in mm.MeshParts)
                    {
                        mmp.Effect = effect;
                    }
                }
            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content. If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools. Otherwise, just
        /// unload ResourceManagementMode.Manual content. Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                content.Unload();
            }
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            if (Keyboard.GetState().IsKeyDown(Keys.Escape) || GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            if (Keyboard.GetState().IsKeyDown(Keys.Left))
            {
                world = world * Matrix.CreateRotationY((float)(-turningSpeed * gameTime.ElapsedGameTime.TotalSeconds));
            }
            if (Keyboard.GetState().IsKeyDown(Keys.Right))
            {
                world = world * Matrix.CreateRotationY((float)(turningSpeed * gameTime.ElapsedGameTime.TotalSeconds));
            }
            if (Keyboard.GetState().IsKeyDown(Keys.Up))
            {
                world = world * Matrix.CreateRotationX((float)(-turningSpeed * gameTime.ElapsedGameTime.TotalSeconds));
            }
            if (Keyboard.GetState().IsKeyDown(Keys.Down))
            {
                world = world * Matrix.CreateRotationX((float)(turningSpeed * gameTime.ElapsedGameTime.TotalSeconds));
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            // uncomment this to see what the z buffer prevents
            //graphics.GraphicsDevice.RenderState.CullMode = CullMode.None;
            //graphics.GraphicsDevice.RenderState.DepthBufferEnable = false;

            // uncomment this to show the triangles
            //graphics.GraphicsDevice.RenderState.FillMode = FillMode.WireFrame;

            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            //Standard-Beleuchtung
            //foreach (ModelMesh mm in model.Meshes)
            //{
            //    foreach(Effect e in mm.Effects)
            //    {
            //        BasicEffect be = (BasicEffect)e;;
            //        be.LightingEnabled = true;
            //        be.DirectionalLight0.DiffuseColor = new Vector3(1.0f, 1.0f, 1.0f);
            //        be.DirectionalLight0.SpecularColor = new Vector3(1.0f, 1.0f, 1.0f);
            //        be.DirectionalLight0.Direction = Vector3.Normalize(new Vector3(1.0f, 1.0f, 0.0f));
            //        be.DirectionalLight0.Enabled = true;
            //        be.SpecularColor = new Vector3(1.0f, 1.0f, 1.0f);
            //        be.DiffuseColor = new Vector3(1.0f, 0.0f, 0.0f);
            //        be.AmbientLightColor = new Vector3(0.0f,0.0f,0.0f);
            //        be.EmissiveColor = new Vector3(0.0f, 0.0f, 0.0f);
            //        be.SpecularColor = Vector3.One;
            //        be.SpecularPower = 20.0f;
            //        be.TextureEnabled = false;
            //        be.World = world;
            //        be.View = Matrix.CreateLookAt(new Vector3(0.0f, 0.0f, 10.0f), Vector3.Zero, Vector3.Up);
            //        be.Projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(50.0f),
            //            graphics.GraphicsDevice.Viewport.Width / (float)graphics.GraphicsDevice.Viewport.Height,
            //            0.1f, 100.0f);
            //    }
            //    mm.Draw();
            //}

            // Rendern mit eigenen Shadern
            Matrix view = Matrix.CreateLookAt(new Vector3(0.0f, 0.0f, 10.0f), Vector3.Zero, Vector3.Up);
            Matrix projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(50.0f),
                graphics.GraphicsDevice.Viewport.Width / (float)graphics.GraphicsDevice.Viewport.Height,
                0.1f, 100.0f);
            effect.Parameters["WorldViewProj"].SetValue(world * view * projection);
            effect.Parameters["World"].SetValue(world);
            effect.Parameters["WorldIT"].SetValue(Matrix.Transpose(Matrix.Invert(world)));
            effect.Parameters["ViewI"].SetValue(Matrix.Invert(view));
            effect.Parameters["DirToLight"].SetValue(Vector3.Normalize(new Vector3(-1.0f, 1.0f, 1.0f)));
            foreach (ModelMesh mm in model.Meshes)
            {
                mm.Draw();
            }

            base.Draw(gameTime);
        }
    }
}